﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using C1.Win.C1FlexGrid;

namespace SlidePuzzle
{
    public partial class Puzzle : UserControl
    {
        bool _win = false;

        public Puzzle()
        {
            InitializeComponent();
            _btnStart.BackgroundImage = Image.FromFile("../../Resources/Img_Start.jpeg");
            _btnStart.BackgroundImageLayout = ImageLayout.Stretch;

            _btnExit.BackgroundImage = Image.FromFile("../../Resources/Img_End.jpeg");
            _btnExit.BackgroundImageLayout = ImageLayout.Stretch;
        }

        private void _btnStart_Click(object sender, EventArgs e)
        {
            _win = false;

            gridPuzzle.BeginUpdate();
            //Get all the pieces in a list
            List<Image> listPieces = new List<Image>();
            listPieces.Add(Image.FromFile("../../Resources/01.jpg"));
            listPieces[0].Tag = "01";
            listPieces.Add(Image.FromFile("../../Resources/21.jpg"));
            listPieces[1].Tag = "21";
            listPieces.Add(Image.FromFile("../../Resources/10.jpg"));
            listPieces[2].Tag = "10";
            listPieces.Add(Image.FromFile("../../Resources/00.jpg"));
            listPieces[3].Tag = "00";
            listPieces.Add(Image.FromFile("../../Resources/22.jpg"));
            listPieces[4].Tag = "22";
            listPieces.Add(Image.FromFile("../../Resources/02.jpg"));
            listPieces[5].Tag = "02";
            listPieces.Add(Image.FromFile("../../Resources/11.jpg"));
            listPieces[6].Tag = "11";
            listPieces.Add(Image.FromFile("../../Resources/20.jpg"));
            listPieces[7].Tag = "20";
            //Fill the grid with pieces, randomly
            int piece = 0;
            for (int col = 0; col < 3; col++)
                for (int row = 0; row < 3; row++)
                {
                    if (!(row == 0 && col == 1))
                    {
                        gridPuzzle.SetCellImage(row, col, listPieces[piece]);
                        gridPuzzle.SetUserData(row, col, listPieces[piece].Tag);
                        piece++;
                    }
                }
            gridPuzzle.Cols[0].ImageAlign = ImageAlignEnum.Stretch;
            gridPuzzle.Cols[1].ImageAlign = ImageAlignEnum.Stretch;
            gridPuzzle.Cols[2].ImageAlign = ImageAlignEnum.Stretch;

            gridPuzzle.EndUpdate();
            //Start the stopwatch
            stopwatch1.Start();

            _btnStart.Enabled = false;
        }

        private void _btnExit_Click(object sender, EventArgs e)
        {
            if(!_win)
            {
                //Better luck next time
                ResultScreen resultScreen = new ResultScreen();
                (resultScreen.Controls["pictureBox1"] as PictureBox).Image = Image.FromFile("../../Resources/Lose.jpeg");
                resultScreen.Text = "See you later! Better luck next time...";
                (resultScreen.Controls["pictureBox1"] as PictureBox).SizeMode = PictureBoxSizeMode.StretchImage;
                resultScreen.FormClosed += ResultScreen_FormClosed;
                resultScreen.Show();
            }
            else
            {
                ((Form)this.TopLevelControl).Close();
            }
        }

        private void ResultScreen_FormClosed(object sender, FormClosedEventArgs e)
        {
            //Time to exit
            ((Form)this.TopLevelControl).Close();
        }

        private void gridPuzzle_MouseClick(object sender, MouseEventArgs e)
        {
            //Player is trying to move pieces
            HitTestInfo hti = gridPuzzle.HitTest(e.Location);
            int row = hti.Row;
            int col = hti.Column;

            gridPuzzle.BeginUpdate();
            if (gridPuzzle.GetCellImage(row, col) != null)
            {
                //To Above
                if (row > 0 && gridPuzzle.GetCellImage(row - 1, col) == null)
                {
                    gridPuzzle.SetCellImage(row - 1, col, gridPuzzle.GetCellImage(row, col));
                    gridPuzzle.SetUserData(row - 1, col, gridPuzzle.GetUserData(row, col));
                    gridPuzzle.SetCellImage(row, col, null);
                    gridPuzzle.SetUserData(row, col, null);
                }
                //To Below
                else if (row < 2 && gridPuzzle.GetCellImage(row + 1, col) == null)
                {
                    gridPuzzle.SetCellImage(row + 1, col, gridPuzzle.GetCellImage(row, col));
                    gridPuzzle.SetUserData(row + 1, col, gridPuzzle.GetUserData(row, col));
                    gridPuzzle.SetCellImage(row, col, null);
                    gridPuzzle.SetUserData(row, col, null);
                }
                //To Left
                else if (col > 0 && gridPuzzle.GetCellImage(row, col - 1) == null)
                {
                    gridPuzzle.SetCellImage(row, col - 1, gridPuzzle.GetCellImage(row, col));
                    gridPuzzle.SetUserData(row, col - 1, gridPuzzle.GetUserData(row, col));
                    gridPuzzle.SetCellImage(row, col, null);
                    gridPuzzle.SetUserData(row, col, null);
                }
                //To Right
                else if (col < 2 && gridPuzzle.GetCellImage(row, col + 1) == null)
                {
                    gridPuzzle.SetCellImage(row, col + 1, gridPuzzle.GetCellImage(row, col));
                    gridPuzzle.SetUserData(row, col + 1, gridPuzzle.GetUserData(row, col));
                    gridPuzzle.SetCellImage(row, col, null);
                    gridPuzzle.SetUserData(row, col, null);
                }
                if (row == 1 && col == 2)
                {
                    //Now, check if all the remaining pieces are in their correct positions
                    if (CheckResult())
                    {
                        //Complete picture
                        gridPuzzle.SetCellImage(row, col, Image.FromFile("../../Resources/12.jpg"));
                        //Congratulate
                        ResultScreen resultScreen = new ResultScreen();
                        resultScreen.Text = "Cogratulations!";
                        (resultScreen.Controls["pictureBox1"] as PictureBox).Image = Image.FromFile("../../Resources/Win.jpeg");
                        (resultScreen.Controls["pictureBox1"] as PictureBox).SizeMode = PictureBoxSizeMode.StretchImage;
                        resultScreen.Show();
                        _win = true;
                        _btnStart.Enabled = true;
                    }
                }
            }
            gridPuzzle.EndUpdate();
        }

        private bool CheckResult()
        {
            bool? win = null;
            for(int col = 0; col < 3; col++)
            {
                if (win == null)
                {
                    for (int row = 0; row < 3; row++)
                    {
                        if (row == 1 && col == 2)
                        {
                            //All the pieces are in their correct places
                            win = true;
                            break;
                        }
                        else if (Convert.ToInt32(gridPuzzle.GetUserData(row, col)) != Convert.ToInt32(row.ToString() + col.ToString()))
                        {
                            //At least one piece is not in its correct place
                            win = false;
                            break;
                        }
                    }
                }
                else
                    break;
            }                
            return Convert.ToBoolean(win);
        }

        private void Puzzle_Load(object sender, EventArgs e)
        {
            //Set styling information
            int totalHeight = gridPuzzle.Height;
            for (int row = 0; row < 3; row++)
                gridPuzzle.Rows[row].Height = totalHeight / 3;
            gridPuzzle.Height += 4;
            gridPuzzle.Styles.Highlight.BackColor = Color.White;
            gridPuzzle.AllowEditing = false;
            gridPuzzle.AllowSorting = AllowSortingEnum.None;
        }
    }
}